# TapiocaStor Enterprise Backup
# Copyright 2002 The TapiocaStor Group
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License (included as file COPYING) for more details.

require 'mkmf'

dir_config("aes")

create_makefile('aes')

